<?php

namespace Intergo\InventoryClient;

use Intergo\BaseClient\Responses\BaseResponseInterface;

abstract class AbstractInventoryService
{
    private InventoryClient $client;
    protected string $secret;

    /**
     * InventoryService constructor.
     *
     * @param $baseUrl
     * @param $traceId
     * @param $site
     * @param $secret
     */
    public function __construct($baseUrl, $traceId, $site, $secret)
    {
        $this->secret = $secret;
        $this->client = new InventoryClient($baseUrl, $traceId, $site);
    }

    /**
     * @param $endpoint
     * @param array $urlParams
     * @param array $bodyParams
     * @return BaseResponseInterface
     */
    protected function executeAuth($endpoint, array $urlParams = [], array $bodyParams = [])
    {
        $result = $this->client->executeAuth($endpoint, $this->secret, $urlParams, $bodyParams);
        return $this->response($endpoint, $result);
    }

    /**
     * @param string $action
     * @param BaseResponseInterface $result
     * @return BaseResponseInterface
     */
    abstract public function response(string $action, BaseResponseInterface $result);

    /**
     * @return BaseResponseInterface
     */
    public function getAllCountries(): BaseResponseInterface
    {
        return $this->executeAuth(__FUNCTION__);
    }

    /**
     * @param $code
     * @return BaseResponseInterface
     */
    public function getCountryCities($code): BaseResponseInterface
    {
        return $this->executeAuth(__FUNCTION__, compact('code'));
    }

    /**
     * @return BaseResponseInterface
     */
    public function getUSStates(): BaseResponseInterface
    {
        return $this->executeAuth(__FUNCTION__);
    }

    /**
     * @param $code
     * @return BaseResponseInterface
     */
    public function getStateCities($code): BaseResponseInterface
    {
        return $this->executeAuth(__FUNCTION__, compact('code'));
    }

    /**
     * @return BaseResponseInterface
     */
    public function getInventoryAvailableCountries(): BaseResponseInterface
    {
        return $this->executeAuth(__FUNCTION__);
    }

    /**
     * @return BaseResponseInterface
     */
    public function getNumbersForUser(): BaseResponseInterface
    {
        return $this->executeAuth(__FUNCTION__);
    }

    /**
     * @return BaseResponseInterface
     */
    public function getNumberTypes(): BaseResponseInterface
    {
        return $this->executeAuth(__FUNCTION__);
    }

    /**
     * @return BaseResponseInterface
     */
    public function getKeywordExamples(): BaseResponseInterface
    {
        return $this->executeAuth(__FUNCTION__);
    }
}
